﻿/*
 * Copyright (C) 2011 by Intelsoft Kazakstan LTD.
 *	
 * Author Ivan Keberlein <ikeberlein@intelsoft.com>
 */
 
System.Gadget.settingsUI = "settings.html";
var testData = {
	contract : 'IS014-4454',
	status : 'online',
	billing_model : 'credit',
	balance: -57600,
	remaining_days: 5,
	traffic: {
		ibytes: 47*1024*1024,
		obytes: 36*1024*1024
	},
	clientIP: '127.1.1.1'
};

var updateInterval = 1800; // secs between updates
var updateCount = 0;
var updateTmrId;

//var dataUrl = 'https://my.10g.kz/informer.php?debug=1&tip=109.238.162.226';
var dataUrl = 'https://my.10g.kz/informer.php?foobar=x';
var xmlReq;

$(document).ready(function() {
	refresh();
});

function getLocalizedString(key) {
	return key;
}

function getDataUrl() {
	var now = new Date(),
		result = dataUrl + '&stamp=' + now.getTime();
	return result; 
}

function refresh() {
	$('#container').addClass('loading');

	xmlReq = new XMLHttpRequest();
    xmlReq.open("GET", getDataUrl(), true);
	xmlReq.onreadystatechange = handleStateChange;
	xmlReq.timeout = handleTimeout;
    xmlReq.send(null);
}

function scheduleUpdate() {
	setTimeout(function(){
		$('#container').removeClass('loading');
	}, 800);

	updateTmrId = setTimeout(refresh, updateInterval * 1000);
}

function handleTimeout() {
	displayError();
	scheduleUpdate();
}

function handleStateChange() {
	if (xmlReq.readyState != 4 /* ! complete */)
		return;

	scheduleUpdate();

//	updateUI(testData);
//	return;
	
	if (xmlReq.status != 200 && xmlReq.status != 304) {
		displayError();
	}

	// now fill data object
    var data = { };
    
    var response = findNodeByTag(xmlReq.responseXML, "response", true);
	
    if (response == null)
        return displayError();
    
    if ((data.contract = getNodeValue(response, "contract", true)) == null)
        return displayError();
        
    if ((data.billing_model = getNodeValue(response, "billing-model", true)) == null)
        return displayError();
    
    if ((data.status = getNodeValue(response, "status", true)) == null)
        return displayError();

    if ((data.clientIP = getNodeValue(response, "client-ip", true)) == null)
        return displayError();
    
    if (data.billing_model == 'prepaid')
        data.remaining_days = getNodeValue(response, "remaining-days-count", false);
    
    data.balance = getNodeValue(response, "balance", false);
 
    var traffic = findNodeByTag(response, "traffic", false);
    if (traffic) {
        data.traffic = { 
            ibytes : getNodeValue(traffic, "ibytes", false),
            obytes : getNodeValue(traffic, "obytes", false)
        };
    } else {
		data.traffic = {
			
		};
	}
    
    updateUI(data);
}

// Find node by tag name
function findNodeByTag(elt, tag, required) {
    var result;
    
    for (child = elt.firstChild; child != null; child = child.nextSibling) {
        if (child.nodeName == tag)
            return child;
    }
    
    if (required) {
        alert("Invalid XML document: <" + tag + "> not found");
    }

    return null;
}

function getNodeValue(elt, tag, is_required) {
    var node = findNodeByTag(elt, tag, is_required);
    
    if (node && node.firstChild) {
        return node.firstChild.nodeValue;
    }
    
    return null;
}

// format number value like this: XXX XXX.XX
function formatNumber(value) {
    var parts,
	collect = []; // symbols added here in reverse order (right-to-left)
		
    if (isNaN(value))
        value = 0.00;

    parts = value.toString().split('.');

    // format fraction part
    if (parts.length > 1) { // do we have fraction part at all ?
        if (parts[1].length < 2)
            collect.push('0');

        collect.push(parts[1].substr(0, 2)); // 
    } else {
        collect.push('00');
    }
    // decimal separator
    collect.push('.');

    // format integer part
    var idx, len = parts[0].length, debugstr = '';
    for (idx = 0; idx < len; idx++) {
        if (idx != 0 && idx % 3 == 0) // add thousands separator
            collect.push(' ');
        collect.push(parts[0].charAt(len - idx - 1));
    }

    return collect.reverse().join('');
}

// format value as money
function formatMoney(value) {
	return formatNumber(value) + ' тг.';
}	

// format traffic value into human readable form
// add unit suffixes (B, Kb, Mb, Gb, Tb) to reduce number of digits
function formatTrafficValue(value) {
    var units = ['Kb', 'Mb', 'Gb', 'Tb'];
    var unit = 'B';
    var i;

    if (value == undefined)
        return null;
    
    for (i = 0; i < units.length; i++) {
        if (value < 1024.0)
            break;
        unit = getLocalizedString(units[i]);
        value /= 1024.0;
    }
    
    value *= 100;
    value = Math.round(value) / 100;
    
    return formatNumber(value) + ' ' + unit;
}

// update UI item identified by id with value
function updateUIItem(elt, value, formatter) {
	if (value != undefined) {
		if (formatter != undefined)
			value = formatter(value);
	} else {
		value = '-.--';
	}
	
	$('#' + elt).text(value);
}

// indicate error
function displayError() {
    $('#stateIndicator').attr('class', 'state-error');

    updateUIItem('txtContract', 'Ошибка связи');
    updateUIItem('txtBalance' , null);
    updateUIItem('txtDays'    , null);
    updateUIItem('txtITraffic', null);
    updateUIItem('txtOTraffic', null);
    updateUIItem('txtClientIP', null);
}

// set state for state indicator
var stateIndicatorOldState, stateIndicatorFlashTmrId;
function setStateIndicator(state, title) {
	var cssKlass;
	
	if (state == stateIndicatorOldState)
		return;

	stateIndicatorOldState = state;
	if (state != 'warn-critical' && stateIndicatorFlashTmrId != undefined) {
		clearInterval(stateIndicatorFlashTmrId);
		stateIndicatorFlashTmrId = undefined;
		$('#stateIndicator').show();
	}
	
	switch (state) {
	case 'online':
		cssKlass = 'state-on';
		break;
		
	case 'warn':
		cssKlass = 'state-warning';
		break;
		
	case 'warn-critical':
		cssKlass = 'state-warning';
		stateIndicatorFlashTmrId = setInterval(stateIndicatorFlasher, 500);
		break;
		
	default:
		cssKlass = 'state-critical';
		break;
	}
	
	$('#stateIndicator').attr('class', cssKlass);
	if (title != undefined) {
		$('#stateIndicator').attr('title', title);
	} else {
		$('#stateIndicator').removeAttr('title');
	}
}

function stateIndicatorFlasher() {
	$('#stateIndicator').toggle();
}

// update whole UI
function updateUI(data) {
	$('body').attr('class', 'billing-model-' + data.billing_model);

    updateUIItem('txtContract', data.contract);
    updateUIItem('txtBalance' , data.balance, formatMoney);
    updateUIItem('txtClientIP', data.clientIP);
    updateUIItem('txtITraffic', data.traffic.ibytes, formatTrafficValue);
    updateUIItem('txtOTraffic', data.traffic.obytes, formatTrafficValue);
	
    if (data.billing_model == 'prepaid') {
        updateUIItem('txtDays', data.remaining_days);
    }

	if (data.status == 'online') {
		if (data.billing_model == 'prepaid' && data.remaining_days <= 3) {
			if (data.remaining_days < 2) {
				setStateIndicator('warn-critical');
			} else {
				setStateIndicator('warn', 
					'Внимание ! Средств на счету осталось примерно на ' 
					+ data.remaining_days 
					+ ((data.remaining_days == 1) ? ' день' : ' дня') 
					+ '. Пополните баланс.'
				);
			}
		} else {
			setStateIndicator('online');
		}
		$('#container').removeClass('contract-offline');
	} else {
		setStateIndicator('offline', 'Договор отключён, необходимо пополнить баланс.');
		$('#container').addClass('contract-offline');
	}
	
    if (data.balance < 0) {
		$('#txtBalance').addClass('negative');
    } else {
		$('#txtBalance').removeClass('negative');
    }
	
}
